# Generated by Django 3.2.14 on 2022-07-11 15:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("notifications_api_common", "0003_migrate_api_root"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="notificationsconfig",
            name="api_root",
        ),
    ]
