# Generated by Django 3.2.14 on 2022-08-10 09:50

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("zgw_consumers", "0015_auto_20220307_1522"),
        ("notifications_api_common", "0004_remove_notificationsconfig_api_root"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="subscription",
            name="config",
        ),
        migrations.AlterField(
            model_name="notificationsconfig",
            name="notifications_api_service",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"api_type": "nrc"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="zgw_consumers.service",
                verbose_name="notifications api service",
            ),
        ),
    ]
