# Generated by Django 3.2.12 on 2022-12-14 06:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("notifications_api_common", "0006_auto_20221213_0214"),
        ("notifications_api_common", "0007_auto_20221206_0414"),
    ]

    operations = []
