# Generated by Django 3.2.15 on 2022-12-13 02:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("notifications_api_common", "0005_auto_20220810_0950"),
    ]

    operations = [
        migrations.AlterField(
            model_name="notificationsconfig",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="subscription",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]
